import { Store, GetterTree, MutationTree, ActionTree, Dispatch as VuexDispatch, DispatchOptions, CommitOptions } from 'vuex';
declare type Not<T, M> = T extends M ? never : T;
declare type StateObject = Not<Record<string, any>, Function>;
declare type StateFunction = Not<() => unknown | any, Record<string, any>>;
declare type State = StateObject | StateFunction;
declare type StateType<T extends State> = T extends () => any ? ReturnType<T> : T;
declare type MergedFunctionProcessor<T extends () => any, O> = Parameters<T>[1] extends undefined ? (options?: O) => ReturnType<T> : Parameters<T>[1] extends NonNullable<Parameters<T>[1]> ? (payload: Parameters<T>[1], options?: O) => ReturnType<T> : (payload?: Parameters<T>[1], options?: O) => ReturnType<T>;
declare type GettersTransformer<T extends Record<string, any>> = Readonly<{
    [P in keyof T]: ReturnType<T[P]>;
}>;
declare type MutationsTransformer<T extends Record<string, any>> = {
    [P in keyof T]: MergedFunctionProcessor<T[P], CommitOptions>;
};
declare type ActionTransformer<T extends Record<string, any>> = {
    [P in keyof T]: MergedFunctionProcessor<T[P], DispatchOptions>;
};
declare type ModuleTransformer<T, O = string> = T extends NuxtModules ? {
    [P in keyof T]: MergedStoreType<T[P] & BlankStore, O>;
} : {};
interface BlankStore {
    state: {};
    getters: {};
    mutations: {};
    actions: {};
    modules: {};
}
interface NuxtStore {
    state: State;
    getters: Record<string, any>;
    mutations: Record<string, any>;
    actions: Record<string, any>;
    modules: NuxtModules;
}
declare type NuxtModules = Record<string, Partial<NuxtStore>>;
interface NuxtStoreInput<T extends State, G, M, A, S extends {
    [key: string]: Partial<NuxtStore>;
}> {
    namespaced?: boolean;
    state: T;
    getters?: G;
    mutations?: M;
    actions?: A;
    modules?: S;
}
declare type MergedStoreType<T extends NuxtStore, K = string> = ('state' extends K ? StateType<T['state']> : {}) & ('getters' extends K ? GettersTransformer<T['getters']> : {}) & ('mutations' extends K ? MutationsTransformer<T['mutations']> : {}) & ('actions' extends K ? ActionTransformer<T['actions']> : {}) & ('modules' extends K ? ModuleTransformer<T['modules']> : {});
declare type StoreParameter<T extends () => any> = Parameters<T>[1] extends undefined ? never : Parameters<T>[1];
interface Commit<T extends Record<string, () => any>> {
    <P extends keyof T>(mutation: P, payload: StoreParameter<T[P]>, options?: DispatchOptions): ReturnType<T[P]>;
    <P extends keyof T>(mutation: StoreParameter<T[P]> extends never ? P : never, options?: CommitOptions): ReturnType<T[P]>;
}
export declare type ActionContext<T extends Required<NuxtStore>> = {
    state: StateType<T['state']>;
    getters: {
        [P in keyof T['getters']]: ReturnType<T['getters'][P]>;
    };
    commit: Commit<T['mutations']>;
    dispatch: VuexDispatch;
    rootState: any;
    rootGetters: any;
};
export declare type RootStateHelper<T extends Required<NuxtStore>> = StateType<T['state']> & ModuleTransformer<T['modules'], 'state'>;
export declare type RootGettersHelper<T extends Required<NuxtStore>> = GettersTransformer<T['getters']> & ModuleTransformer<T['modules'], 'getters'>;
export declare const getStoreType: <T extends Record<string, any>, G, M, A, S extends Record<string, Partial<NuxtStore>>>(store: NuxtStoreInput<T, G, M, A, S>) => {
    actionContext: ActionContext<NuxtStoreInput<T, G, M, A, S> & BlankStore>;
    rootState: RootStateHelper<NuxtStoreInput<T, G, M, A, S> & BlankStore>;
    rootGetters: RootGettersHelper<NuxtStoreInput<T, G, M, A, S> & BlankStore>;
    storeInstance: ActionContext<NuxtStoreInput<T, G, M, A, S> & BlankStore> & Store<StateType<T>>;
};
export declare const getAccessorType: <T extends Record<string, any>, G extends GetterTree<StateType<T>, any>, M extends MutationTree<StateType<T>>, A extends ActionTree<StateType<T>, any>, S extends Record<string, Partial<NuxtStore>>>(store: Partial<NuxtStoreInput<T, G, M, A, S>>) => MergedStoreType<Partial<NuxtStoreInput<T, G, M, A, S>> & BlankStore, string>;
export declare const useAccessor: <T extends Record<string, any>, G extends GetterTree<StateType<T>, any>, M extends MutationTree<StateType<T>>, A extends ActionTree<StateType<T>, any>, S extends Record<string, Partial<NuxtStore>>>(store: Store<any>, input: Partial<NuxtStoreInput<T, G, M, A, S>>, namespace?: string | undefined) => MergedStoreType<Partial<NuxtStoreInput<T, G, M, A, S>> & BlankStore, string>;
export declare const getAccessorFromStore: (pattern: any) => (store: Store<any>) => MergedStoreType<Partial<NuxtStoreInput<Record<string, any>, GetterTree<Record<string, any>, any>, MutationTree<Record<string, any>>, ActionTree<Record<string, any>, any>, Record<string, Partial<NuxtStore>>>> & BlankStore, string>;
export declare const getterTree: <S, T extends GetterTree<StateType<S>, any>>(_state: S, tree: T) => T;
export declare const mutationTree: <S, T extends MutationTree<StateType<S>>>(_state: S, tree: T) => T;
interface ActionHandler<T extends NuxtStore> {
    (this: Store<StateType<T['state']>>, injectee: ActionContext<T>, payload?: any): any;
}
interface ModifiedActionTree<T extends NuxtStore> {
    [key: string]: ActionHandler<T>;
}
interface NormalisedActionHandler<T extends ActionHandler<any>> {
    (this: Store<any>, ...args: Parameters<T>): ReturnType<T>;
}
declare type NormalisedActionTree<T extends ModifiedActionTree<any>> = {
    [P in keyof T]: NormalisedActionHandler<T[P]>;
};
export declare const actionTree: <S extends Record<string, any>, G extends GetterTree<StateType<S>, any>, M extends MutationTree<StateType<S>>, T extends ModifiedActionTree<Required<NuxtStoreInput<S, G, M, {}, {}>>>>(_store: NuxtStoreInput<S, G, M, {}, {}>, tree: T) => NormalisedActionTree<T>;
export {};
